-------------------------------------------------------------------------------
-- Cloud Serpent by Lockslap, US-Bleeding Hollow
-- http://ba-guild.com
-------------------------------------------------------------------------------
local CloudSerpent		= LibStub("AceAddon-3.0"):NewAddon("Cloud Serpent", "AceEvent-3.0", "AceConsole-3.0")
local L					= LibStub("AceLocale-3.0"):GetLocale(CloudSerpent.name, false)
if not TomTom then return end

CloudSerpent.name		= GetAddOnMetadata("CloudSerpent", "Title")
CloudSerpent.version	= GetAddOnMetadata("CloudSerpent", "Version")
CloudSerpent.db			= {}

local curZone, subZone
local waypoints	= {}
local defaults	= {
	char = {
		enabled = true,
	}
}

-- zones that will activate the waypoints
local zones = {
	[L["Mistveil Sea"]]		= true,
	[L["Windward Isle"]]	= true,
	[L["The Arboretum"]]	= true,
}

local repString			= L["You are |cff%s%d/%d %s (%d%%)|r with the |cff1784d1Order of the Cloud Serpent|r."]
local requiredString	= L["You require |cff1784d1%d|r %s%s, at |cff1784d1%d|r rep each, to reach Exalted"]
local iconString		= "\124T%s:12\124t"
local enabledString		= ("|cff00ff00%s|r"):format(L["Enabled"])
local disabledString	= ("|cffff0000%s|r"):format(L["Disabled"])

local repStandings 		= {
	[1]	= L["Hated"],
	[2]	= L["Hostile"],
	[3]	= L["Unfriendly"],
	[4]	= L["Neutral"],
	[5]	= L["Friendly"],
	[6]	= L["Honored"],
	[7]	= L["Revered"],
	[8]	= L["Exalted"],
}
local factionColors		= {
	[1] = {r = 0.8, g = 0.3, b = 0.22},
	[2] = {r = 0.8, g = 0.3, b = 0.22},
	[3] = {r = 0.75, g = 0.27, b = 0},
	[4] = {r = 0.9, g = 0.7, b = 0},
	[5] = {r = 0, g = 0.6, b = 0.1},
	[6] = {r = 0, g = 0.6, b = 0.1},
	[7] = {r = 0, g = 0.6, b = 0.1},
	[8] = {r = 0, g = 0.6, b = 0.1},
}

local function ClearWaypoints()
	if #waypoints == 0 then return end
	for _, point in pairs(waypoints) do
		TomTom:RemoveWaypoint(point)
	end
	waypoints = {}
	TomTom:ReloadWaypoints()
end

function CloudSerpent:HandleChatCommand(args)
	local command = self:GetArgs(args)
	if not command or command == "" or command == "toggle" then
		CloudSerpent.db.char.enabled = not CloudSerpent.db.char.enabled
		CloudSerpent:Print((L["Waypoints have been %s"]):format(CloudSerpent.db.char.enabled == true and enabledString or disabledString))
		if not CloudSerpent.db.char.enabled and #waypoints > 0 then
			ClearWaypoints()
		elseif CloudSerpent.db.char.enabled == true then
			curZone = GetZoneText()
			subZone = GetSubZoneText()
			CloudSerpent:UpdateWaypoints(curZone, subZone)
		end
	elseif command == "status" then
		local _, link, _, _, _, _, _, _, _, texture, _ = GetItemInfo(89155)	-- onyx egg
		local icon = iconString:format(texture)
		local realValue, realMax, percent, numEggs = 0, 0, 0, 0
		local rep, race = 500, UnitRace("player")
		local guildLevel = IsInGuild() and GetGuildLevel() or nil
		local _, _, standing, barMin, barMax, barValue, _, _, _, _, _, _, _ = GetFactionInfoByID(1271)	-- cloud serpent's id is 1271
		local hexColor = ("%02x%02x%02x"):format(factionColors[standing].r * 255, factionColors[standing].g * 255, factionColors[standing].b * 255)
		
		-- first determine if player is human
		if race:lower() == "human" then
			-- player has diplomacy
			rep = rep + (rep * 0.10)
		end
		
		-- now determine if player's guild has mr. popularity ranks 1 or 2
		if guildLevel and guildLevel >= 4 and guildLevel < 12 then
			-- rank 1
			rep = rep + (rep * 0.05)
		elseif guildLevel and guildLevel >= 12 then
			-- rank 2
			rep = rep + (rep * 0.10)
		end
		
		-- determine the rep values displayed on he reputation page, and the percentage completed
		realValue = barValue - barMin
		realMax = barMax - barMin
		percent = math.floor(realValue / realMax * 100)
		
		-- now determine how many more eggs are required to hit exalted
		numEggs = math.ceil((barMax - barValue) / rep)
		
		-- now print everything
		self:Print(repString:format(hexColor, realValue, realMax, repStandings[standing], percent))
		if standing ~= 8 then then
			self:Print(requiredString:format(numEggs, icon, link, rep))
		end
	end
end

function CloudSerpent:UpdateWaypoints(zone, sub)
	if not self.db.char.enabled then return end
	-- only need waypoints in the mistveil sea, windward isle, and the arboretum
	if zone == L["The Jade Forest"] and zones[sub] == true and #waypoints == 0 then
		for _, coord in pairs(self.coordinates) do
			-- add the waypoint to tomtom
			waypoints[#waypoints + 1] = TomTom:AddZWaypoint(6, 6, coord.x, coord.y, L["Onyx Egg Location"], false, false, false, false, false, true)
		end
		--self:Print((L["Loaded |cff1784d1%d|r waypoints."]):format(#self.coordinates))
	elseif zone ~= L["The Jade Forest"] or not zones[sub] then
		ClearWaypoints()
	end
end

function CloudSerpent:ZoneChanged(event, ...)
	-- these will need to update regardless of which event fired
	curZone = GetZoneText()
	subZone = GetSubZoneText()
	if self.db.char.enabled == true then
		self:UpdateWaypoints(curZone, subZone)
	end
end

function CloudSerpent:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("CloudSerpentDB", defaults)
	-- set tomtom's arrival distance to 5
	TomTom.profile.arrow.arrival = 5
	-- make sure the arrow is enabled
	TomTom.profile.arrow.enable = true
end

function CloudSerpent:OnEnable()
	-- register the chat commands to toggle the addon
	self:RegisterChatCommand("cloudserpent", "HandleChatCommand")
	self:RegisterChatCommand("cs", "HandleChatCommand")
end

function CloudSerpent:OnDisable()
	self:UnregisterChatCommand("cloudserpent")
	self:UnregisterChatCommand("cs")
end

-- tracked events
CloudSerpent:RegisterEvent("ZONE_CHANGED", "ZoneChanged")
CloudSerpent:RegisterEvent("ZONE_CHANGED_INDOORS", "ZoneChanged")
CloudSerpent:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChanged")

-- the coordinates of known egg locations
CloudSerpent.coordinates = {
	{x = 65.08, y = 26.20},
	{x = 66.64, y = 28.69},
	{x = 67.12, y = 24.94},
	{x = 65.18, y = 24.77},
	{x = 67.05, y = 33.81},
	{x = 66.61, y = 31.58},
	{x = 66.54, y = 32.20},
	{x = 63.09, y = 29.31},
	{x = 69.65, y = 31.64},
	{x = 70.53, y = 28.99},
	{x = 61.89, y = 30.20},
	{x = 66.18, y = 33.43},
	{x = 62.40, y = 32.36},
	{x = 65.63, y = 25.07},
	{x = 64.18, y = 23.92},
	{x = 63.39, y = 21.62},
	{x = 62.98, y = 21.95},
	{x = 62.37, y = 32.84},
	{x = 62.17, y = 31.75},
	{x = 62.19, y = 31.21},
	{x = 67.02, y = 32.68},
	{x = 62.46, y = 28.77},
	{x = 62.21, y = 29.29},
	{x = 62.00, y = 29.54},
	{x = 63.73, y = 21.90},
	{x = 62.30, y = 28.12},
	{x = 62.71, y = 29.78},
	{x = 66.43, y = 29.55},
	{x = 65.18, y = 24.77},
	{x = 65.24, y = 23.91},
	{x = 70.33, y = 28.63},
	{x = 67.06, y = 30.25},
}